#!/usr/bin/env python

from collections import OrderedDict

list_gene = [["ND2", 3333, 4400, "-"],
             ["ND1", 4472, 5452, "-"],
             ["COI", 6447, 8000, "+"]
            ]

### https://www.ncbi.nlm.nih.gov/Taxonomy/Utils/wprintgc.cgi?chapter=tgencodes#SG2
geneticCodeMTvert = {
            "TTT" : "F",
            "TTC" : "F",
            "TTA" : "L",
            "TTG" : "L",
            "CTT" : "L",
            "CTC" : "L",
            "CTA" : "L",
            "CTG" : "L",
            "ATT" : "I",
            "ATC" : "I",
            "ATA" : "M",
            "ATG" : "M",
            "GTT" : "V",
            "GTC" : "V",
            "GTA" : "V",
            "GTG" : "V",
            "TCT" : "S",
            "TCC" : "S",
            "TCA" : "S",
            "TCG" : "S",
            "CCT" : "P",
            "CCC" : "P",
            "CCA" : "P",
            "CCG" : "P",
            "ACT" : "T",
            "ACC" : "T",
            "ACA" : "T",
            "ACG" : "T",
            "GCT" : "A",
            "GCC" : "A",
            "GCA" : "A",
            "GCG" : "A",
            "TAT" : "Y",
            "TAC" : "Y",
            "TAA" : "*",
            "TAG" : "*",
            "CAT" : "H",
            "CAC" : "H",
            "CAA" : "Q",
            "CAG" : "Q",
            "AAT" : "N",
            "AAC" : "N",
            "AAA" : "K",
            "AAG" : "K",
            "GAT" : "D",
            "GAC" : "D",
            "GAA" : "E",
            "GAG" : "E",
            "TGT" : "C",
            "TGC" : "C",
            "TGA" : "W",
            "TGG" : "W",
            "CGT" : "R",
            "CGC" : "R",
            "CGA" : "R",
            "CGG" : "R",
            "AGT" : "S",
            "AGC" : "S",
            "AGA" : "*",
            "AGG" : "*",
            "GGT" : "G",
            "GGC" : "G",
            "GGA" : "G",
            "GGG" : "G"}

geneticCodeMTinvert = {
            "TTT" : "F",
            "TTC" : "F",
            "TTA" : "L",
            "TTG" : "L",
            "CTT" : "L",
            "CTC" : "L",
            "CTA" : "L",
            "CTG" : "L",
            "ATT" : "I",
            "ATC" : "I",
            "ATA" : "M",
            "ATG" : "M",
            "GTT" : "V",
            "GTC" : "V",
            "GTA" : "V",
            "GTG" : "V",
            "TCT" : "S",
            "TCC" : "S",
            "TCA" : "S",
            "TCG" : "S",
            "CCT" : "P",
            "CCC" : "P",
            "CCA" : "P",
            "CCG" : "P",
            "ACT" : "T",
            "ACC" : "T",
            "ACA" : "T",
            "ACG" : "T",
            "GCT" : "A",
            "GCC" : "A",
            "GCA" : "A",
            "GCG" : "A",
            "TAT" : "Y",
            "TAC" : "Y",
            "TAA" : "*",
            "TAG" : "*",
            "CAT" : "H",
            "CAC" : "H",
            "CAA" : "Q",
            "CAG" : "Q",
            "AAT" : "N",
            "AAC" : "N",
            "AAA" : "K",
            "AAG" : "K",
            "GAT" : "D",
            "GAC" : "D",
            "GAA" : "E",
            "GAG" : "E",
            "TGT" : "C",
            "TGC" : "C",
            "TGA" : "W",
            "TGG" : "W",
            "CGT" : "R",
            "CGC" : "R",
            "CGA" : "R",
            "CGG" : "R",
            "AGT" : "S",
            "AGC" : "S",
            "AGA" : "S",
            "AGG" : "S",
            "GGT" : "G",
            "GGC" : "G",
            "GGA" : "G",
            "GGG" : "G"}


def readFasta_dict(InfileNameFN):
    Infile = open(InfileNameFN, "r")
    seqDictFN  = OrderedDict()
    for Line in Infile:
        Line = Line.rstrip("\n")
        if Line[0] == ">":
            Name            = Line
            seqDictFN[Name] = ""
        else:
            seqDictFN[Name] += Line
    Infile.close()
    return seqDictFN

def revcom(seqFN):
    seqFN = seqFN.upper()
    seqFN = seqFN.replace('A', 't')
    seqFN = seqFN.replace('T', 'a')
    seqFN = seqFN.replace('C', 'g')
    seqFN = seqFN.replace('G', 'c')
    seqFN = seqFN[::-1]
    return seqFN.upper()

def translation(dna):
    dna = dna.upper()
    protein = ""
    for codon in splitDna(dna):
        aaChr = geneticCodeMTinvert.get(codon, "X")     ## <= GeneticCode
        protein = protein + aaChr + "  "
    return protein

def splitDna(dna):
    codons = []
    for start in range(0, len(dna)-2, 3):
        codons.append(dna[start:start+3])
    return(codons)


####
InfileName = "020_mafOut.txt"
seqDict = readFasta_dict(InfileName)

for ele in list_gene:
    name_gene = ele[0]
    start = ele[1] - 1
    end = ele[2]
    strand = ele[3]
    print("name_gene", name_gene)
    #print("start_gene", start_gene)
    #print("end_gene", end_gene)
    #print("strand_gene", strand_gene)

    out = open(name_gene + ".txt", "w")
    for name_otu, seq in seqDict.items():
        seq2 = seq[int(start):int(end)]
        if strand == "-":
            seq2 = revcom(seq2)
        out.write(name_otu + "\n")
        out.write(seq2 + "\n")
        aaSeq = translation(seq2)
        out.write(aaSeq + "\n")
    out.close()
